;*******************************************************                                                     *
;*                                                     *
;*  Project    : Caps Catcher                          *
;*               for PC AT Compatibles and up          *
;*						       *
;*  File Name  : CAPSFW02.ASM                          *
;*                                                     *
;*  Revision   : 2                                     *
;*						       *
;*  Date       : December 1997			       *
;*						       *
;*  Author     : Mark Beattie                          *
;*                                                     *
;*  Designer   : Joe Raine                             *
;*                                                     *
;*   (c) Copyright  QUEST Electronics Pty Limited      *
;*                  ACN 003 501 282                    *
;*       also trading as QUESTRONIX & Quest AV         *
;*      P O Box 548, WAHROONGA, NSW, 2076, AUSTRALIA   *
;*           Email - questav@questronix.com.au         *
;*                                                     *
;*    NOTES:-                                          *
;*    1. This file and any files generated by any      *
;*       means from this file may be used freely       *
;*       for private, non-profit purposes ONLY.        *
;*    2. This file and any files generated by any      *
;*       means from this file may not be used, in      *
;*       part or whole for any profit-making           *
;*       enterprise without obtaining our express      *
;*       written permission prior to such use.         *
;*    3. This file may be modified and distributed     *
;*       by any means for any non-profit purpose/s     *
;*       provided that this header remains intact      *
;*       and is ALWAYS part of such file/s.            *
;*                                                     *
;*                                                     *
;*    Target Microcontroller:- 			       *
;*               Microchip PIC16C54-HS/P               *
;*		 WatchDog disabled                     *
;*               Code protect off                      *
;*******************************************************
;
	LIST    P = 16C54, n = 66
	INCLUDE         <P16C5X.INC>

PIC54   equ     1FFH    ; Define Reset Vector

;******************************************************************************
; Define constants
;
Same    equ     1
MSB     equ     7
CLK	equ	2	; CLK pin - Bit 0 of PORT B
DR      equ     3       ; Receive Pin - Bit 1 of PORT B )
SW	equ	0	; Switch pin - Bit 0 of PORTB
;
;******************************************************************************
; Reserve RAM bytes
;
	ORG     08H     ; Dummy Origin
;
RcvReg  RES     1       ; Data received
Count   RES     1       ; Counter
Bytes	RES     1	; No. of bytes received
Caps	RES	1	; Caps lock flag
Fflg	RES	1	; F0 flag
ED	RES	1	; ED flag
Bpflg	RES	1	; Beep subroutine flag
Bpn	RES	1	; Flag to set 3 chrs or 30
LcfF	RES	1	; Flag for flashing LED in LC mode
Shtdwn	RES	1	; Shut-down flag
lp1	RES	1	; Loops
lp2	RES	1	;
lp3	RES	1	;
L1	RES	1	;
L2	RES	1	;
L3	RES	1	;
;
;******************************************************************************
	ORG     0
;******************************************************************************
; AT scancodes - set 2, a-z in order.
;******************************************************************************
Table	addwf	PCL,1		
	nop
	retlw	01CH
	retlw	032H
	retlw	021H
	retlw	023H
	retlw	024H
	retlw	02BH
	retlw	034H
	retlw	033H
	retlw	043H
	retlw	03BH
	retlw	042H
	retlw	04BH
	retlw	03AH
	retlw	031H
	retlw	044H
	retlw	04DH
	retlw	015H
	retlw	02DH
	retlw	01BH
	retlw	02CH
	retlw	03CH
	retlw	02AH
	retlw	01DH
	retlw	022H
	retlw	035H
	retlw	01AH
;******************************************************************************
; Start - main program. Monitors CLK and DTA lines for data, and checks switch.
;******************************************************************************
Start:
	clrf	RcvReg		; Clear all bits of RcvReg
	btfsc	Bpn,0		;
	goto	Wait		;
Lcf_L2	movlw	050H		;
	movwf	L2		; Load L2 with 80
Lcf_L1	movlw	0FFH		;
	movwf	L1		; Load L1 with 255
;
Wait	btfss	PORTB,CLK	; Check if CLK line is high
	goto	Pctst		;   Go to PC > KYBD test if not
	btfsc	PORTB,DR	; Check if DATA line is low
	goto	SWtest		;   If not, test switch
	goto	Rcvr		;   Go to KYBD > PC reception if so
;
Pctst	btfss	PORTB,DR	; Check if DATA line is high man
	goto	Wait		;   If both lines low, go back to start
	goto	RcvPC2K		;   Go to PC > KYBD reception if so
;
;******************************************************************************
; SWtest - acts on input from push-button
;******************************************************************************
SWtest:
	btfss	PORTB,SW	; Check if switch is open
	goto	SWLD3		;   If closed, check if held down
				;    for 1 sec, or released
	btfsc	Shtdwn,0	; Check if shut down
	goto	Wait		;   Go back to start if so
	btfss	Bpn,0		; Check if long-count mode is on
	goto	Wait		;   Go back to start if not
Lcf_1	decfsz	L1,1		; Decrement counter 1
	goto	Wait		;   Loop if > 0
Lcf_2	decfsz	L2,1		; Decrement counter 2
	goto	Lcf_L1		;   Loop if > 0
	comf	LcfF,1		; Toggle LED status
	btfss	LcfF,0		; Check if status is on
	goto	Lcf_off		;   Turn LED off if not
	bcf	PORTA,2		;   Turn LED on if so,
	goto	Lcf_L2		;    go back to start.
Lcf_off	bsf	PORTA,2		; Turn LED off,
	goto	Lcf_L2		;  go back to start.
;
; Determine if switch is held down or released quickly
SWLD3	btfsc	Shtdwn,0	; Check for shut-down
	goto	Wake		;  Wake up if so
	movlw	04H		;  Otherwise:
	movwf	L3		; Load L3 with 4
SWLD2	movlw	0FFH
	movwf	L2		; Load L2 with 255 (0FFH)
SWLD1	movlw	0FFH
	movwf	L1		; Load L1 with 255 (0FFH)
SWLP1	decfsz	L1,1		; Decrement counter 1
	goto	SWLT1		;  IF > 0 check
	goto	SWLP2		;  Go to next loop if = 0
SWLT1	call	SWtst_2		; Check if switch has been released
	goto	SWLP1		;  Continue loop if not
SWLP2	decfsz	L2,1		; Decrement counter 2
	goto	SWLT2		;  If > 0 check
	goto	SWLP3		;  Go to next loop if = 0
SWLT2	call	SWtst_2		; Check if switch has been released
	goto	SWLD1		;  Repeat loop1 if not
SWLP3	decfsz	L3,1		; Decrement counter 3
	goto	SWLT3		;  If > 0 check
	goto	SW_end		;  End loop if = 0
SWLT3	call	SWtst_2		; Check if switch has been released
	goto	SWLD2		;  Repeat loops 1 & 2 if not
SWtst_2	btfsc	PORTB,SW	; Check if switch is closed
	goto	SW_exit		;   If not, exit.
	retlw	0		;   Return if so
;
; Switch was pushed and released quickly
SW_exit	btfss	Caps,0		; Check if caps-lock is on
	goto	Wait		;   If not, go back to start
	bsf	Bpflg,0		; Set flag to tell Beep to return
	call	Beep		; Beep
	bcf	Bpflg,0		; Clear Beep flaggy thing
	btfss	Bpn,0		; Check if already in long-count mode
	goto	set_lc		;   Switch to long-count mode if not already.
	clrf	Bpn		;   If already in lc mode: cancel it,
	bcf	PORTA,2		;     turn on caps-lock LED,
	goto	Start		;     go back to start.
set_lc	bsf	Bpn,0		; Set LC mode flag
;	bsf	PORTA,2		; Turn off caps led
	goto	Start		; Go back to start
;
; Switch was held down for 1.2 sec:
; Shut down
SW_end	bsf	Bpflg,0		; Set flag to tell Beep to return
	bcf	PORTA,2		; Turn caps LED on
	call	Beep		; Beep once...
	bsf	PORTA,2		; Turn caps LED off
	call	Shrt_D		; Short delay before second beep
	bcf	PORTA,2		; Turn caps LED on
	call	Beep		; Beep again...
;	bcf	Bpflg,0		; Clear Beep flaggy thing
; The above line is made redundant by the fact that the next action after
; being shut-down is to beep as it wakes up. The routine that calls 'Beep'
; cancels this flag anyway.
	bsf	PORTA,2		; Turn caps LED off
	bsf	PORTA,3		; Turn power LED off
	bcf	Bpn,0		; Cancel long-count mode
End_t	btfss	PORTB,SW	; Check if switch is still being held down
	goto	End_t		;   If so, wait until it is released
	goto	Debounce1	; Debounce switch
DBret1	bsf	Shtdwn,0	; Set shut-down flag
	goto	Start		; Back to begining
;
; Wake - return from shut-down mode
Wake:
	bsf	Bpflg,0		; Tell beep to come back here...
	bcf	PORTA,2		; Turn caps LED on
	call	Beep		; Beep...
	clrf	Bpflg		; Set beep back to normal operation
	clrf	Shtdwn		; Clear shut-down flag
	call	Shrt_D		;
	call	Shrt_D		; De-bounce switch
	call	Shrt_D		;
	goto	cfchk		; Re-start
;
; Short delay routine
Shrt_D	movlw	0FFH		;
	movwf	L2		;
SDL	movlw	0FFH		;
	movwf	L1		;
SW_del	decfsz	L1,1		; Short delay
	goto	SW_del		;
	decfsz	L2,1		;
	goto	SDL		;
	retlw	0		; Return
;******************************************************************************
; Receiver - KYBD > PC
; Enters routine with start bit (low) in DR
; Data is clocked on falling edge of clock.
;******************************************************************************
Rcvr	call	Waitneg		;
	call	Waitpos		; Start bit.
;
	movlw	08H		; 8 DATA bits
	movwf	Count
R_next  bcf     STATUS,C
	call	Waitneg		;
	rrf     RcvReg,Same     ; Rotate right; LSB first
	btfsc   PORTB,DR
	bsf     RcvReg,MSB      ; Set bit if PORTA,DR is high man
	call    Waitpos		; Wait for next bit
	decfsz  Count,Same	;
	goto    R_next		; Loop for 8 bits
;
	call	Waitneg		;
	call	Waitpos		; Parity bit
;
	call	Waitneg		; Last falling-edge clocks stop-bit
;
	goto	endtest		; Last few routines from this section
				; had to be moved because of #@#$ page
				; boundaries, dang it!
;******************************************************************************
; Receiver - KYBD < PC
; Recieves byte from host
;******************************************************************************
RcvPC2K:
;
WaitDL	btfsc	PORTB,DR	; Wait for DATA line to come down
	goto	WaitDL		;
;
	call	Waitpos		;
	call	Waitneg		; Start bit
;
	movlw   08H		; 8 DATA bits
	movwf   Count
R_nxt	bcf	STATUS,C
	call	Waitpos		;
	rrf     RcvReg,Same     ; Rotate right; LSB first
	btfsc   PORTB,DR
	bsf     RcvReg,MSB      ; Set bit if PORTA,DR is high man.
	call    Waitneg		; Wait for next bit
	decfsz  Count,Same	;
	goto    R_nxt		; Loop for 8 bits
;
	call	Waitpos		;
	call	Waitneg		; Parity bit
;
	call	Waitpos		;
	call	Waitneg		; Stop bit
;
;	call	Waitpos		; Keyboard Acknowledge
;
WaitDH	btfss	PORTB,DR	; Wait for data-line to go high
	goto	WaitDH
;
	call	Waitneg		; Precautionary check for rogue pulse
;
	goto	F0chk		; Check for codes
;******************************************************************************
; Routines to test byte for keys/commands
;******************************************************************************
F0chk:
	movlw	0F0H		; Check for F0
	subwf	RcvReg,0
	btfss	STATUS,Z
	goto	FFchk		;   Not F0 - test for FF: Reset
	bsf	Fflg,0		;   Is F0 - Set F0 flag,
	goto	Line_test	;     Get next byte
;
;
FFchk:
	movlw	0FFH		; Check for FF
	subwf	RcvReg,0
	btfss	STATUS,Z
	goto	EDchk		;   Not FF - test for 0EDH: Status
	clrf	Caps		;   Is FF - Clear Caps-Lock flag,
	clrf	Bpn		;    cancel long-count mode,
	clrf	Bytes		;    clear bytes counter,
	bsf	PORTA,2		;    turn off indicator LED,
	goto	Line_test	;    Go back
;
;
EDchk:
	movlw	0EDH		; Check for ED - status. Turns CL led on/off
	subwf	RcvReg,0
	btfss	STATUS,Z	; Check for 0EDH
	goto	CL_on		;   Not ED - test for CL on codes
	bsf	ED,0		;   Is ED - set ED flag,
	goto	Line_test	;   go back for next byte.
;
;
CL_on:
	btfss	ED,0		; Check if PC has sent 0EDH
	goto	Codechk		;   Check for keys if not
	movlw	0FAH
	subwf	RcvReg,0
	btfsc	STATUS,Z	; Check if last byte was ACK
	goto	Line_test	;   Get next byte if so
	btfsc	RcvReg,2	; Get CL status
	goto	Cset		;   Turn CL on
	clrf	Caps		;   Turn CL off
	goto	cfchk
Cset	bsf	Caps,0		; Set CL flag
;
cfchk	btfss	Caps,0		; Check if caps is on -
	goto	Clrcaps
	btfss	Shtdwn,0	; Skip next line if shut down
	bcf	PORTA,2		; Turn Caps led on if so
	goto	Cend
Clrcaps	bsf	PORTA,2		; Turn Caps led off if not
	clrf	Bpn		; and cancel long-count mode.
;
Cend	clrf	ED		; Clear ED flag
	goto	Line_test	; Back for next byte
;
;
Codechk:
	btfss	Fflg,0		; Check for F0 flag
	goto	Line_test	;   Go to start if clear
	btfsc	Shtdwn,0	; Check if shut down
	goto	Line_test	;   Go to start if so
	btfss	Caps,0		; Check if caps lock is on
	goto	Line_test	;   Go back if not
	movlw	1BH		; Test for scancodes
	movwf	Count		; Load Count with 27 (1BH)
TabStrt	decfsz	Count,1		; Decrement counter
	goto	Tabnxt		;   Go to table look up if count>0
	goto	Tabend		;   End table if count=0
Tabnxt	movf	Count,W		; Load Count into W
	call	Table		; Get result from look-up table
	subwf	RcvReg,0
	btfss	STATUS,Z	; Check if RcvReg is a valid code
	goto	TabStrt		;   Check next table value if not
	goto	KeySet		;   Go to KeySet if so
Tabend	btfss	Bpn,0		; RcvReg didn't match any codes
	clrf	Bytes		;  Clear key counter if not in LC mode;
	clrf	Fflg		;  clear F0 flag,
	clrf	ED		;  clear ED flag,
	goto	Line_test	;  only gets here if no key was found.
;******************************************************************************
; Subroutines
;
;******************************************************************************
; Waitneg - Waits for negative transition of clock
Waitneg	btfsc	PORTB,CLK
	goto	Waitneg
	retlw	0
;
; Waitpos - Waits for positive transition of clock
Waitpos	btfss	PORTB,CLK	;
	goto	Waitpos		;
	retlw	0
;
; KeySet - program is sent here if a valid key is found.
; Increments counter and decides whether to beep or not
KeySet:
	clrf	Fflg		; Clear F0 flag
	btfss	Caps,0		; Check for Caps lock flag
	goto	Line_test	; Go back if Caps are not on
	incf	Bytes,1		; Increment byte counter
	movlw	03H		; Load no. of keys with 3
	btfsc	Bpn,0		; Check for LC mode
	movlw	01EH		;   If so, load no. of keys with 30
	subwf	Bytes,0		;
	btfss	STATUS,Z	; Check if < 3 (or 30) keys have been pressed
	goto	Line_test	;   Go back if so
	goto	Beep		;   Beep if 3 keys have been pressed
;
; Line_test - After a byte is recieved control is sent here
; Test CLK and DTA to determine if next byte is from host or kybd
Line_test:
	clrf	RcvReg
LT_1	btfsc	PORTB,CLK	; Check if clock has gone high
	goto	Start		;   Return to start if so
	btfss	PORTB,DR	; Check if Data has gone low
	goto	RcvPC2K		;   Recieve byte from host if so
	goto	LT_1
;
; Beep - It beeps.
Beep:
	movlw	0FFH		; Beep routine.
	movwf	Count		; Load count with 255
Loop0	movlw	2		;
	movwf	lp1		; Load lp1 with 2
	bcf	PORTA,3		; Turn beeper on
Loop2	movlw	4CH		; 
	movwf	lp2		; Load lp2 with 76 (4CH)
Loop1	decfsz	lp2,1		; Delay for 114uS (T/2 - f=4.4 kHz)
	goto	Loop1		;   Loop until lp2=0
	bsf	PORTA,3		; Turn beeper off
	decfsz	lp1,1		; Loop again - T = 2*114uS = 227uS
	goto	Loop2		; f = 1 / T = 4.4 kHz
	decfsz	Count,1		; Duration = count x T = 50mS
	goto	Loop0		;   Loop until count=0
;
	clrf	Bytes		; Clear Bytes counter
	bcf	PORTA,3		; Turn power LED back on
	btfss	Bpflg,0		; Check if subroutine flag is set
	goto	Start		;   If not, bo back to start
	retlw	0		;   If so, return
;
; Last routines from KBD < PC reception that had to be moved
; because of page boundaries
endtest:
	movlw	01BH		
	movwf	Count		; Load counter with 27 (1BH)
W_40	decfsz	Count,1		; Delay for 40uS (27*1.5uS)
	goto	W_40
	btfsc	PORTB,CLK	; Check if CLK is low...
	call	Waitneg		;   If not, wait till it is
	goto	F0chk		; Check code
;
; Debounce routine(s) - moved because of page boundaries
Debounce1:
DBld1	movlw	0FFH
	movwf	Count
DBounce	decfsz	Count,1		; De-bounce switch:
	goto	DBtest1		; When switch is relesed it must stay
	goto	DBexit1		; high for ~400uS (~0.5mS)
DBtest1	btfss	PORTB,SW
	goto	DBld1
	goto	DBounce
DBexit1:
	goto	DBret1
;******************************************************************************
; Main
;******************************************************************************
main    movlw	0		; 00000000 B
	tris    PORTA           ; PA2 - Caps LED, PA3 - Beeper
	movlw	0FH		; 00001111 B
	tris	PORTB		; RB0 - switch, RB2 - CLK, RB3 - DTA
	clrf	PORTA		; Turn on LED's
;
Clean	bsf	PORTA,2		; Turn caps-lock LED off
	bcf	PORTA,3		; Turn red (power) LED on
	clrf	Caps		; Clear Caps-Lock flag
	clrf	Fflg		; Clear F0 flag
	clrf	Bytes		; Clear bytes counter
	clrf	Bpn		; Clear LC mode flag
	clrf	Shtdwn		; Clear shut-down flag
;
	goto	Start		; begin reception
;
;
;******************************************************************************
; Reset vector:
;******************************************************************************
	ORG	PIC54
	goto	main
;
	END
